<?php
/**
 * @brief randomPost, a plugin for Dotclear 2
 *
 * @package Dotclear
 * @subpackage Plugins
 *
 * @author julien mudry and contributors
 *
 * @copyright julien mudry
 * @copyright GPL-2.0 https://www.gnu.org/licenses/gpl-2.0.html
 */
declare(strict_types=1);

namespace Dotclear\Plugin\randomPost;

use dcCore;
use dcNamespace;
use Dotclear\Core\Backend\Notices;
use Dotclear\Core\Backend\Page;
use Dotclear\Core\Process;
use Dotclear\Helper\Html\Form\Div;
use Dotclear\Helper\Html\Form\Form;
use Dotclear\Helper\Html\Form\Input;
use Dotclear\Helper\Html\Form\Label;
use Dotclear\Helper\Html\Form\Li;
use Dotclear\Helper\Html\Form\Para;
use Dotclear\Helper\Html\Form\Radio;
use Dotclear\Helper\Html\Form\Submit;
use Dotclear\Helper\Html\Form\Text;
use Dotclear\Helper\Html\Form\Ul;
use Dotclear\Helper\Html\Html;
use Exception;

class Manage extends Process
{
    public static function init(): bool
    {
        return self::status(My::checkContext(My::MANAGE));
    }

    public static function process(): bool
    {
        if (!self::status()) {
            return false;
        }

        $settings = My::settings();
        if (is_null($settings->randomPost_category)) {
            try {
                // Add default settings values if necessary
                $settings->put(
                    'randomPost_category',
                    My::ALL_CAT_ID,
                    dcNamespace::NS_STRING,
                    'Category limitation when selecting a random post (only from template functions)'
                );
            } catch (Exception $e) {
                dcCore::app()->error->add($e->getMessage());
            }
        }

        if (!empty($_POST)) {
            try {
                $settings->put('randomPostURL', $_POST['randomPostURL'], dcNamespace::NS_STRING, 'Prefix for the URL redirecting to a random post');
                $settings->put('randomPostStaticURL', $_POST['randomPostStaticURL'], dcNamespace::NS_STRING, 'Prefix for the URL displaying a random post');
                $settings->put('randomPost_category', $_POST['randomPost_category'], dcNamespace::NS_STRING, 'Category limitation when selecting a random post (only from template functions)');

                dcCore::app()->blog->triggerBlog();
                Notices::addSuccessNotice(__('Configuration successfully updated'));
                My::redirect();
            } catch (Exception $e) {
                dcCore::app()->error->add($e->getMessage());
            }
        }

        return true;
    }

    public static function render(): void
    {
        if (!self::status()) {
            return;
        }

        $head = Page::jsPageTabs('options');

        Page::openModule(My::name(), $head);
        echo Page::breadcrumb(
            [
                Html::escapeHTML(dcCore::app()->blog->name) => '',
                __('Random Post plugin') => '',
            ]
        );
        echo Notices::getNotices();

        self::renderOptions();
        self::renderUsage();
        self::renderChangelog();

        Page::closeModule();
    }

    private static function renderOptions(): void
    {
        $settings = My::settings();

        echo (new Div('options'))
        ->title(__('Options'))
        ->class(['multi-part'])
        ->items([
            (new Form('frmsettings'))
                ->action(dcCore::app()->admin->getPageURL())
                ->method('post')
                ->fields([
                    (new Para())->items([
                        (new Input('randomPostURL'))
                            ->size(30)
                            ->maxlength(128)
                            ->value(Html::escapeHTML($settings->randomPostURL))
                            ->label((new Label(__('Prefix for the URL redirecting to a random post:'), Label::OUTSIDE_LABEL_BEFORE))),
                    ]),
                    (new Para())->items([
                        (new Input('randomPostStaticURL'))
                            ->size(30)
                            ->maxlength(128)
                            ->value(Html::escapeHTML($settings->randomPostStaticURL))
                            ->label((new Label(__('Prefix for the URL displaying a random post:'), Label::OUTSIDE_LABEL_BEFORE))),
                    ]),
                    (new Para())->items([
                        (new Input('randomPost_category'))
                            ->size(30)
                            ->maxlength(128)
                            ->value(Html::escapeHTML($settings->randomPost_category))
                            ->label((new Label(__('Category limitation when selecting a random post (only from template functions):'), Label::OUTSIDE_LABEL_BEFORE))),
                    ]),

                    (new Para())->items([
                        (new Submit(['frmsubmit']))
                            ->value(__('Save')),
                        ... My::hiddenFields(),
                    ]),
                ])
        ])
        ->render();
    }

    private static function renderUsage(): void
    {
        echo (new Div('usage'))
        ->title(__('Usage'))
        ->class(['multi-part'])
        ->items([
            (new Text('h3'))->text(__('Usage')),
            (new Para())->items([(new Text)->text(sprintf(__("In your template, just create a link with %s as a destination. When your visitors'll click on that link, they'll be redirect to a random post from your blog."), '<kbd>{{tpl:RandomRedirURL}}</kbd>'))]),
            (new Para())->items([(new Text)->text(sprintf(__("Or, if you prefer, you can also use it as a widget. The link will be enclosed in a %s with class %s and displayed as a %s."), '<kbd>&lt;div&gt;</kbd>', '<kbd>randompost</kbd>', '<kbd>&lt;li&gt;</kbd>'))]),
            (new Text('h3'))->text(__('Advanced usage')),
            (new Para())->items([
                new Text('strong', __('Note:') . ' '),
                (new Text())->text(__('I advise you strongly <strong>not</strong> to use these advanced techniques on every page of your blog.') . ' '),
                (new Text())->text(__("Doing so will result in two more SQL requests for every page views, which might slow down your site to a crawl. You've been warned.")),
            ]),
            (new Para())->items([(new Text())->text(__('You can display both an excerpt and the title from the post, either with the widget options or the template functions.'))]),
            (new Ul())->items([
                (new Li())->items([
                    (new Text())->text(__('Using the widgets options:')),
                    (new Ul(null, 'dl'))->items([
                        (new Text('dt'))->text(__('Title (optional):')),
                        (new Text('dd'))->text(__("Title of the block displayed in the sidebar. Leave it empty if you use the redirection and don't want a heading title before it.")),
                        (new Text('dt'))->text(__('Link text:')),
                        (new Text('dd'))->text(__('Link text if you use redirection.')),
                        (new Text('dt'))->text(__('Use redirection')),
                        (new Text('dd'))->text(__("Use a URL for redirection as configured in the plugins' settings. This is the prefered method, as the needed database requests will only be executed when your users click on the link.")),
                        (new Text('dt'))->text(__('Display an excerpt from the post')),
                        (new Text('dd'))->text(__("Will display the excerpt from the post, or the post's content if there's no excerpt.")),
                        (new Text('dt'))->text(__('Excerpt maximum length (in characters):')),
                        (new Text('dd'))->text(__('Limit the displayed excerpt to a certain maximum length. Use 0 if you want to display the full excerpt.')),
                        (new Text('dt'))->text(__('HTML tags allowed in the output:')),
                        (new Text('dd'))->text(sprintf(__('Specify the HTML tags you want to allow in the excerpt. You can put multiple tags, for example: %s'), '<kbd>&lt;img&gt;&lt;a&gt;</kbd>')),
                        (new Text('dt'))->text(__('Restrict to category')),
                        (new Text('dd'))->text(__('With this option, you can restrict the posts to one category only.').' '.__('Choose "All categories" if you don\'t want this limit.')),
                        (new Text('dt'))->text(__('Display on:')),
                        (new Text('dd'))->text(__('Choose on which pages the widget should be displayed.')),
                        (new Text('dt'))->text(__('Content only')),
                        (new Text('dd'))->text(__('Do not wrap the widget with a div.')),
                        (new Text('dt'))->text(__('Offline')),
                        (new Text('dd'))->text(__('Do not display the widget at all.')),
                    ])
                ]),
                (new Li())->items([
                    (new Text())->text(__('Using the template functions:')),
                    (new Ul(null, 'dl'))->items([
                        (new Text('dt'))->text('<kbd>{{tpl:RandomRedirURL}}</kbd>'),
                        (new Text('dd'))->text(__('Use this function as the destination of a link.').' '.__('This is the prefered method, as the needed database requests will only be executed when your users click on the link.')),
                        (new Text('dt'))->text('<kbd>&lt;tpl:RandomPost&gt;</kbd>'),
                        (new Text('dd'))->text(__('Start of a template block. It selects a random post, which can then be accessed and displayed with the next functions.')),
                        (new Text('dt'))->text('<kbd>{{tpl:RandomPostURL}}</kbd>'),
                        (new Text('dd'))->text(__('Returns the URL of the selected post.')),
                        (new Text('dt'))->text('<kbd>{{tpl:RandomPostTitle}}</kbd>'),
                        (new Text('dd'))->text(__('Returns the title of the selected post.')),
                        (new Text('dt'))->text('<kbd>{{tpl:RandomPostExcerpt}}</kbd>'),
                        (new Text('dd'))->text(__("Returns the excerpt (or content it there's no excerpt) of the selected post.").' '.
                            sprintf(__('Can take one attribute, %s, which indicates the maximum length for the excerpt displayed.'), '<kbd>max_length</kbd>').' '.
                            sprintf(__('Use 0 for %s to disable this truncation (the default is 200 characters).'), '<kbd>max_length</kbd>'),
                        ),
                    ])
                ])
            ]),
            (new Para())->items([
                (new Text())->text(sprintf(__('Both the %s and %s template functions can take one attribute: %s.'), '<kbd>&lt;tpl:RandomPost&gt;</kbd>', '<kbd>{{tpl:RandomRedirURL}}</kbd>', '<kbd>limit_to_category_id</kbd>').' '),
                (new Text())->text(__('This attribute takes a numeric value: the ID of a category.').' '),
                (new Text())->text(__('If you use that attribute, all the posts randomly selected will belong to the given category.')),
            ]),
            (new Para())->items([
                (new Text('strong'))->text(__('Note:') . ' '),
                (new Text())->text(__('In the widget, the options "Use redirection" and "Display an excerpt from the post" are mutually exclusive.') . ' '),
                (new Text())->text(__('If you check both boxes, the option "Use redirection" will take precedence.')),
            ])
        ])
        ->render();
    }

    private static function renderChangelog(): void
    {
        $releaseText = __('Released on %s');
        echo (new Div('changelog'))
            ->title(__('Changelog'))
            ->class(['multi-part'])
            ->items([
                (new Text('h3'))->text('Version 1.2.0'),
                (new Text('p'))->text(sprintf($releaseText, '2023-10-21')),
                (new Ul())->items([
                    (new Li())->text(__('Rewritten to the standards of Dotclear 2.27.')),
                    (new Li())->text(__('Fixed the widget display to match current standards.')),
                ]),

                (new Text('h3'))->text('Version 1.1.1'),
                (new Text('p'))->text(sprintf($releaseText, '2023-09-04')),
                (new Ul())->items([
                    (new Li())->text(__('Fixed the compatibility with Dotclear 2.27.')),
                    (new Li())->text(__('Added some missing French translations.')),
                ]),

                (new Text('h3'))->text('Version 1.1.0'),
                (new Text('p'))->text(sprintf($releaseText, '2023-02-05')),
                (new Ul())->items([
                    (new Li())->text(__("Added the possibility to edit the URL prefixes in the plugin's admin page.")),
                    (new Li())->text(__('Fixed the bug where installing a new version of the plugin emptied the settings for the URL prefixes.')),
                ]),

                (new Text('h3'))->text('Version 1.0.1'),
                (new Text('p'))->text(sprintf($releaseText, '2023-01-16')),
                (new Ul())->items([
                    (new Li())->text(__('Fixed the invalid link to the post in the widget.')),
                ]),

                (new Text('h3'))->text('Version 1.0.0'),
                (new Text('p'))->text(sprintf($releaseText, '2022-12-25')),
                (new Ul())->items([
                    (new Li())->text(__('Made it compatible with Dotclear version 2.24.')),
                ]),

                (new Text('h3'))->text(__('Older versions')),
                (new Text('p'))->text(__('For older versions, see the CHANGELOG.md file distributed with the plugin.')),
            ])
            ->render();
    }
}
